﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;

namespace HeuristicLab.Services.Hive.DataAccess.Daos {
  public class JobDao : GenericDao<Guid, Job> {
    public JobDao(DataContext dataContext) : base(dataContext) { }

    public override Job GetById(Guid id) {
      return GetByIdQuery(DataContext, id);
    }
    
    public void DeleteByState(JobState state) {
      DataContext.ExecuteCommand(DeleteByStateQueryString, Enum.GetName(typeof(JobState), state));
    }

    public IEnumerable<Job> GetByState(JobState state) {
      return GetByStateQuery(DataContext, state);
    }

    public IEnumerable<Guid> GetJobIdsByState(JobState state) {
      return GetJobIdsByStateQuery(DataContext, state);
    }

    public IEnumerable<Job> GetJobsReadyForDeletion() {
      return GetJobsReadyForDeletionQuery(DataContext);
    }


    #region Compiled queries
    private static readonly Func<DataContext, Guid, Job> GetByIdQuery =
      CompiledQuery.Compile((DataContext db, Guid jobId) =>
        (from job in db.GetTable<Job>()
         where job.JobId == jobId
         select job).SingleOrDefault());
    private static readonly Func<DataContext, JobState, IEnumerable<Job>> GetByStateQuery =
      CompiledQuery.Compile((DataContext db, JobState jobState) =>
        (from job in db.GetTable<Job>()
         where job.State == jobState
         select job).ToList());
    private static readonly Func<DataContext, JobState, IEnumerable<Guid>> GetJobIdsByStateQuery =
      CompiledQuery.Compile((DataContext db, JobState jobState) =>
        (from job in db.GetTable<Job>()
         where job.State == jobState
         select job.JobId).ToList());
    private static readonly Func<DataContext, IEnumerable<Job>> GetJobsReadyForDeletionQuery =
      CompiledQuery.Compile((DataContext db) =>
        (from job in db.GetTable<Job>()
         where job.State == JobState.StatisticsPending
         && (from task in db.GetTable<Task>()
             where task.JobId == job.JobId
             select task.State == TaskState.Finished
              || task.State == TaskState.Aborted
              || task.State == TaskState.Failed).All(x => x)
         select job).ToList());
    #endregion

    #region String queries
    private const string DeleteByStateQueryString = @"
      DELETE FROM [Job]
      WHERE JobState = {0}
    ";
    private const string GetStatisticsPendingJobs = @"
      SELECT DISTINCT j.*
      FROM [Job] j
      WHERE j.JobState = 'StatisticsPending'
      AND 'Calculating' NOT IN (
        SELECT t.TaskState 
        FROM [Task] t 
        WHERE t.JobId = j.JobId)
    ";
    #endregion
  }
}
